<?xml version="1.0" encoding="ISO-8859-1"?><xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:SMLNL="http://www.shef.ac.uk/SpineMLNetworkLayer"  xmlns:SMLLOWNL="http://www.shef.ac.uk/SpineMLLowLevelNetworkLayer" xmlns:SMLCL="http://www.shef.ac.uk/SpineMLComponentLayer" xmlns:NMLEX="http://www.shef.ac.uk/SpineMLExperimentLayer" xmlns:fn="http://www.w3.org/2005/xpath-functions">
<xsl:output method="text" version="1.0" encoding="UTF-8" indent="yes"/>

<!-- INCLUDE THE HELPER TEMPLATES -->
<xsl:include href="SpineML_helpers.xsl"/>

<!-- MAIN TEMPLATE -->
<xsl:template match="/"> <!-- mode="prototypes"-->

<!-- CREATE REFERENCES TO THE FILES USED IN THIS SCRIPT -->
<xsl:variable name="expt_file" select="/"/>
<xsl:variable name="model_file" select="document(concat('../model/', $expt_file//NMLEX:Model/@network_layer_url))"/>
<xsl:variable name="split_model_file" select="document(concat('../model/split_', $expt_file//NMLEX:Model/@network_layer_url))"/>

<!-- MOVE INTO THE ORIGINAL MODEL FILE -->
<xsl:for-each select="$model_file">

<!-- SET UP SOME FIXED VALUES FOR MAXIMUM CONNECTIVITY -->
<xsl:variable name="max_hash_size" select="'80'"/><!-- REPLACE LATER -->
<xsl:variable name="max_hash_columns" select="'3'"/>
<xsl:variable name="max_total_inputs" select="'80'"/><!-- REPLACE LATER -->
<xsl:variable name="inputs_per_port" select="'80'"/><!-- REPLACE LATER -->
<xsl:variable name="number_of_input_ports" select='80'/><!-- REPLACE LATER -->
<xsl:variable name="max_pop_size" select="'100'"/><!-- REPLACE LATER -->


<xsl:for-each select="$split_model_file/SMLLOWNL:SpineML/SMLLOWNL:Population">

<!-- SOME USEFUL VARIABLES -->
<xsl:variable name="node_number" select="position()"/>
<xsl:variable name="curr_split_pop" select="."/>
<xsl:variable name="curr_split_pop_name" select="SMLLOWNL:Neuron/@name"/>
<xsl:variable name="curr_unsplit_pop" select="$model_file//SMLLOWNL:Population[SMLLOWNL:Neuron/@name=substring-before($curr_split_pop/SMLLOWNL:Neuron/@name, '_sub')]"/>
<xsl:variable name="curr_unsplit_name_sanitised" select="translate($model_file//SMLLOWNL:Neuron[@name=substring-before($curr_split_pop/SMLLOWNL:Neuron/@name, '_sub')]/@name, ' -', 'oo')"/>

<!-- GET A LIST OF THE PROJECTIONS THAT TARGET THIS POPULATION-->
<xsl:variable name="proj_inputs" select="/SMLLOWNL:SpineML//SMLLOWNL:Projection[@dst_population=$curr_split_pop_name]/SMLLOWNL:Synapse"/>
<xsl:variable name="proj_inputs_unsplit" select="$model_file//SMLLOWNL:Projection[@dst_population=$curr_unsplit_pop/SMLLOWNL:Neuron/@name]/SMLLOWNL:Synapse"/>

<!-- GET A LIST OF ALL THE COMPONENTS IN THE NRN AND ITS PROJECTIONS -->
<xsl:variable name="all_components" select="$proj_inputs/SMLLOWNL:WeightUpdate/@url | $proj_inputs/SMLLOWNL:PostSynapse/@url | $curr_split_pop/SMLLOWNL:Neuron/@url"/>
<xsl:variable name="all_unsplit_components" select="$proj_inputs_unsplit/SMLLOWNL:WeightUpdate/@url | $proj_inputs_unsplit/SMLLOWNL:PostSynapse/@url | $curr_unsplit_pop/SMLLOWNL:Neuron/@url"/>

<!-- #ALIAS FOR CURRENT POPULATION -->
#alias <xsl:value-of select="$curr_unsplit_name_sanitised"/><xsl:text> </xsl:text><xsl:value-of select="$node_number"/>

num_neurons_actual = <xsl:value-of select="$curr_split_pop/SMLLOWNL:Neuron/@size"/>;

// data for hash tables ///////////////////////
hashCreation={
<!---->
<!-- FOR EACH INPUT POPULATION -->
<xsl:for-each select="$all_unsplit_components/../../SMLLOWNL:WeightUpdate/../../../SMLLOWNL:Neuron | $model_file//SMLLOWNL:Neuron[@name=$all_unsplit_components/../SMLLOWNL:Input/@src]">
	<xsl:variable name="src_switch_value" select="position()"/>
	<xsl:variable name="src_unsplit_name" select="@name"/>
	<!-- FOR EACH SRC SPLIT POPULATION -->
        <xsl:for-each select="$split_model_file/SMLLOWNL:SpineML/SMLLOWNL:Population">
		<!-- GENERATE SRC SPLIT POPULATION NODE NUMBER -->
		<xsl:variable name="src_node_number" select="position()"/>
                <xsl:variable name="src_pop_name" select="SMLLOWNL:Neuron/@name"/>
		<!-- IF POPULATION DERIVED FROM UNSPLIT INPUT POPULATION -->
		<xsl:if test="substring-before($src_pop_name, '_sub')=$src_unsplit_name">
			<!-- IF SRC SPLIT POPULATION IS AN INPUT TO CURR SPLIT POPULATION -->
                        <xsl:if test="count($all_components/../SMLLOWNL:Input[@src=$src_pop_name] | $all_components/../../../../SMLLOWNL:Neuron[@name=$src_pop_name])>0">
				<!-- GET SPLIT INDEX FOR CURRENT SRC -->
                                <xsl:for-each select="$split_model_file//SMLLOWNL:Neuron[substring-before(@name, '_sub')=$src_unsplit_name and (@name=$proj_inputs/../../SMLLOWNL:Neuron/@name or @name=$all_components/../SMLLOWNL:Input/@src)]">
					<xsl:if test="@name=$src_pop_name">
						<xsl:variable name="src_pop_split_index" select="position()"/>
						<!-- ADD A ROW TO THE ARRAY {src_node, switch_value, split_index}-->
						<!---->{<!---->
						<xsl:value-of select="concat($src_node_number ,', ')"/>
						<xsl:value-of select="concat($src_switch_value ,', ')"/>
						<xsl:value-of select="number($src_pop_split_index)-1"/>
						<!---->},
<!---->				</xsl:if>
				</xsl:for-each>
			</xsl:if>
		</xsl:if>
	</xsl:for-each>
</xsl:for-each>

<!-- RIDICULOUSLY COMPLEX XPATH EXPRESSION THAT CALCULATES HOW MAY ARRAY ENTRIES THE ABOVE CODE MAKES! -->
<xsl:variable name="num" select="count($split_model_file//SMLLOWNL:Neuron[substring-before(@name, '_sub')=($all_unsplit_components/../../SMLLOWNL:WeightUpdate/../../../SMLLOWNL:Neuron/@name | $model_file//SMLLOWNL:Neuron[@name=$all_unsplit_components/../SMLLOWNL:Input/@src]/@name) and (@name=$all_components/../SMLLOWNL:Input/@src or @name=$all_components/../../../../SMLLOWNL:Neuron/@name)])"/>

<!-- ADD EXTRA LINES SO THAT THE DIMENSIONALITY IS CORRECT -->
<xsl:variable name="hashline">
	<!---->{<!---->
	<xsl:call-template name="RepeatString">
	  <xsl:with-param name="string" select="'0'" />
	  <xsl:with-param name="divider" select="', '" />
	  <xsl:with-param name="times"  select="$max_hash_columns"/>
	</xsl:call-template>
	<!---->}<!---->
</xsl:variable>
<xsl:call-template name="RepeatString">
  <xsl:with-param name="string" select="$hashline" />
  <xsl:with-param name="divider" select="',&#10;'" />
  <xsl:with-param name="times"  select="$max_total_inputs - $num" />
</xsl:call-template>
};

<!--


-->
// close unused output ports (neuron only) //
portOn = {<!---->
<!-- FOR EACH NEURON ANALOG AND EVENT OUTPUT PORT ON THE CURRENT POPULATION -->
<xsl:for-each select="document($curr_split_pop/SMLLOWNL:Neuron/@url)//SMLCL:AnalogSendPort | document($curr_split_pop/SMLLOWNL:Neuron/@url)//SMLCL:EventSendPort | document($curr_split_pop/SMLLOWNL:Neuron/@url)//SMLCL:ImpulseSendPort">
	<xsl:variable name="curr_port_name" select="@name"/>
	<xsl:choose>
		<!-- IF THAT PORT IS REFERENCED IN A SYNAPSE OR GENERIC INPUT -->
                <xsl:when test="count($curr_split_pop//SMLLOWNL:WeightUpdate[@input_src_port=$curr_port_name] | $split_model_file//SMLLOWNL:Input[@src=$curr_split_pop_name and @src_port=$curr_port_name and not(@src=../../../@dst_population and count(SMLNL:OneToOneConnection)=1 and local-name(..)='PostSynapse')])>0">
			<!-- OPEN PORT -->1, <!---->
		</xsl:when>
		<!-- IF NOT THEN -->
		<xsl:otherwise>
			<!-- CLOSE PORT -->0, <!---->
		</xsl:otherwise>
	</xsl:choose>
</xsl:for-each>

<!-- CALCULATE HOW MANY OUTPUT PORTS THERE ARE -->
<xsl:variable name="total_ports" select="count(document($curr_split_pop/SMLLOWNL:Neuron/@url)//SMLCL:AnalogSendPort | document($curr_split_pop/SMLLOWNL:Neuron/@url)//SMLCL:EventSendPort | document($curr_split_pop/SMLLOWNL:Neuron/@url)//SMLCL:ImpulseSendPort)"/>

<!-- ADD EXTRA LINES SO THAT THE DIMENSIONALITY IS CORRECT -->
<xsl:if test="(number($max_total_inputs)-number($total_ports))>0">
	<xsl:call-template name="RepeatString">
		  <xsl:with-param name="string" select="'0'" />
		  <xsl:with-param name="divider" select="', '" />
		  <xsl:with-param name="times"  select="(number($max_total_inputs)-number($total_ports))" />
	</xsl:call-template>
</xsl:if>
<!---->};
		
// initialise explicit lists //////////////////

// parameters / SVs
// synapse
<xsl:for-each select="$proj_inputs_unsplit">
	<!-- IF IT IS A SYNAPSE -->
        <xsl:apply-templates select="SMLLOWNL:WeightUpdate" mode="initExplicitData">
                <xsl:with-param name="curr_pop_name" select="$curr_unsplit_pop/SMLLOWNL:Neuron/@name"/>
		<xsl:with-param name="curr_split_pop_name" select="$curr_split_pop_name"/>
		<xsl:with-param name="split_model_file" select="$split_model_file"/>
		<xsl:with-param name="n_dst" select="$max_pop_size"/>
		<xsl:with-param name="n_src" select="$max_pop_size"/>
	</xsl:apply-templates>
</xsl:for-each>
		
// postsynapse
<xsl:for-each select="$proj_inputs_unsplit/SMLLOWNL:PostSynapse">
	<!-- GENERATE THE UNIQUE SUFFIX NAME FOR THIS POSTSYNAPSE -->
	<xsl:variable name="suffix">
		<xsl:value-of select="translate(@name,' -','__')"/><!-- NEW SUFFIX -->
	</xsl:variable>
	<xsl:variable name="unsplit_name" select="@name"/>
        <xsl:for-each select="$proj_inputs/SMLLOWNL:PostSynapse[substring-before(@name, '_sub')=$unsplit_name]">
		<xsl:if test="position()=1">
			<!-- FOR EACH UNSPLIT PROPERTY THAT USES AN EXPLICIT LIST -->
			<xsl:for-each select="SMLNL:Property[count(SMLNL:ValueList)=1]">
				<xsl:variable name="curr_prop_name" select="@name"/>
				<xsl:value-of select="@name"/><xsl:value-of select="$suffix"/> = {<!---->
				<!-- FOR EACH SPLIT OF THIS PROPERTY IN THE SPLIT MODEL -->
				<xsl:variable name="curr_prop" select="."/>
				<xsl:call-template name="PropertyExplicitVals121">
					<xsl:with-param name="input" select="."/>
					<xsl:with-param name="length" select="$max_pop_size"/>
				</xsl:call-template>
				<!---->};
<!---->		</xsl:for-each>
		</xsl:if>
	</xsl:for-each>
</xsl:for-each>

// neuron
<xsl:for-each select="$curr_split_pop/SMLLOWNL:Neuron">
	<!-- GENERATE THE UNIQUE SUFFIX NAME FOR THIS POSTSYNAPSE -->
	<xsl:variable name="suffix">
		<xsl:value-of select="translate(@name,' -','__')"/><!-- NEW SUFFIX -->
	</xsl:variable>
	<xsl:variable name="unsplit_name" select="@name"/>
	<!-- FOR EACH UNSPLIT PROPERTY THAT USES AN EXPLICIT LIST -->
	<xsl:for-each select="SMLNL:Property[count(SMLNL:ValueList)=1]">
		<xsl:variable name="curr_prop_name" select="@name"/>
		<xsl:value-of select="@name"/><xsl:value-of select="$suffix"/> = {<!---->
		<!-- FOR EACH SPLIT OF THIS PROPERTY IN THE SPLIT MODEL -->
		<xsl:variable name="curr_prop" select="."/>
		<xsl:call-template name="PropertyExplicitVals121">
			<xsl:with-param name="input" select="."/>
			<xsl:with-param name="length" select="$max_pop_size"/>
		</xsl:call-template>
		<!---->};
<!----></xsl:for-each>
</xsl:for-each>

// connections (synapse)
<xsl:for-each select="$proj_inputs_unsplit">
	<!-- IF IT IS A SYNAPSE -->
        <xsl:apply-templates select="SMLLOWNL:WeightUpdate" mode="initExplicitConns">
                <xsl:with-param name="curr_pop_name" select="$curr_unsplit_pop/SMLLOWNL:Neuron/@name"/>
		<xsl:with-param name="split_model_file" select="$split_model_file"/>
		<xsl:with-param name="curr_split_pop_name" select="$curr_split_pop_name"/>
		<xsl:with-param name="n_dst" select="$max_pop_size"/>
		<xsl:with-param name="n_src" select="$max_pop_size"/>
	</xsl:apply-templates>
</xsl:for-each>


// connections (generic inputs)
<!-- FOR EACH UNSPLIT POSTSYNAPSE -->
<xsl:for-each select="$proj_inputs_unsplit/SMLLOWNL:PostSynapse">
	<xsl:variable name="unsplit_name" select="@name"/>
	<!-- FOR EACH SPLIT POSTSYNAPSE DERIVED FOR THE UNSPLIT POSTSYNAPSE -->
        <xsl:for-each select="$proj_inputs/SMLLOWNL:PostSynapse[substring-before(@name, '_sub')=$unsplit_name]">
		<!-- ONLY USE THE 1ST SPLIT (ALL SPLITS SHOULD BE IDENTICAL) -->
		<xsl:if test="position()=1">
                        <xsl:for-each select="SMLLOWNL:Input">
				<xsl:apply-templates select="." mode="initExplicitConns">
					<xsl:with-param name="curr_dst" select="."/>
					<xsl:with-param name="split_model_file" select="$split_model_file"/>
					<xsl:with-param name="unsplit_model_file" select="$model_file"/>
					<xsl:with-param name="n_dst" select="$max_pop_size"/>
					<xsl:with-param name="n_src" select="$max_pop_size"/>
				</xsl:apply-templates>
			</xsl:for-each>
		</xsl:if>
	</xsl:for-each>
</xsl:for-each>

<!-- FOR EACH INPUT IN THE UNSPLIT NEURON BODY -->
<xsl:for-each select="$curr_unsplit_pop/SMLLOWNL:Neuron/SMLLOWNL:Input">
	<xsl:apply-templates select="." mode="initExplicitConns">
                <xsl:with-param name="curr_dst" select="$curr_split_pop/SMLLOWNL:Neuron"/>
		<xsl:with-param name="split_model_file" select="$split_model_file"/>
		<xsl:with-param name="unsplit_model_file" select="$model_file"/>
		<xsl:with-param name="n_dst" select="$max_pop_size"/>
		<xsl:with-param name="n_src" select="$max_pop_size"/>
	</xsl:apply-templates>
</xsl:for-each>

// setup interrupts:
clock_int: 0
<xsl:if test="not(count($split_model_file//SMLLOWNL:Population[not(count(SMLLOWNL:Projection[@dst_population=$curr_split_pop_name])=0) or $curr_split_pop/SMLLOWNL:Neuron/SMLLOWNL:Input/@src=SMLLOWNL:Neuron/@name])=0)">
event_int: <!---->
</xsl:if>
<xsl:for-each select="$split_model_file//SMLLOWNL:Population[not(count(SMLLOWNL:Projection[@dst_population=$curr_split_pop_name])=0) or $curr_split_pop/SMLLOWNL:Neuron/SMLLOWNL:Input/@src=SMLLOWNL:Neuron/@name]">
	<xsl:variable name="dst_pop_name" select="SMLLOWNL:Neuron/@name"/>
        <xsl:for-each select="$split_model_file//SMLLOWNL:Population">	
        	<xsl:if test="SMLLOWNL:Neuron/@name = $dst_pop_name">
        		<xsl:value-of select="position()"/>
        	</xsl:if>
	</xsl:for-each>
	<xsl:if test="not(position()=last())">
		<!---->, <!---->
	</xsl:if>
</xsl:for-each>

// outputs
<!----><xsl:for-each select="$expt_file//NMLEX:LogOutput[@target=substring-before($curr_split_pop_name,'_sub')]">
		<xsl:variable name="outputName" select="@name"/>
		<xsl:variable name="outputPopName" select="@target"/>
		<xsl:variable name="outputPortName" select="@port"/>
		<xsl:variable name="outputNumber" select="position()"/>
		<xsl:variable name="outputStartTime">
			<xsl:if test="count(@start_time)=1">
				<xsl:value-of select="@start_time"/>
			</xsl:if>
			<xsl:if test="count(@start_time)=0">
				<!---->0<!---->
			</xsl:if>
		</xsl:variable>
		<xsl:variable name="outputEndTime">
			<xsl:if test="count(@end_time)=1">
				<xsl:value-of select="@end_time"/>
			</xsl:if>
			<xsl:if test="count(@end_time)=0">
				<!---->1000000<!---->
			</xsl:if>
		</xsl:variable>
		<!-- if event -->
		<xsl:if test="count(document($model_file//*[@name=$outputPopName]/@url)//SMLCL:EventSendPort[@name=$outputPortName])=1">
#snapshot "spike" 0 1000000 0.000001 "%f %d\n" t current_neuron_index
<!---->
		</xsl:if>
		<!-- if analog -->
		<xsl:if test="count(document($model_file//*[@name=$outputPopName]/@url)//SMLCL:AnalogSendPort[@name=$outputPortName])=1">
		 	<xsl:variable name="suffix_name">
		 		<xsl:for-each select="$model_file//*[@name=$outputPopName]">
					<xsl:value-of select="translate(@name,' -','__')"/><!-- NEW SUFFIX -->
		 		</xsl:for-each>
		 	</xsl:variable>
		 		<xsl:if test="@indices">
					<!---->#log "<!---->
					<xsl:value-of select="$outputName"/>
					<!---->" <!---->
					<xsl:value-of select="$outputStartTime"/>
					<xsl:text> </xsl:text>
					<xsl:value-of select="$outputEndTime"/>
					<!----> 0.000001 "%f<!---->
		 			<xsl:call-template name="write_printf_output">
		 				<xsl:with-param name="indices" select="@indices"/>
		 			</xsl:call-template>
					<!---->\n" t <!---->
		 			<xsl:call-template name="write_indices_output">
		 				<xsl:with-param name="indices" select="@indices"/>
		 				<xsl:with-param name="component" select="$model_file//*[@name=$outputPopName]"/>
		 				<xsl:with-param name="port" select="$outputPortName"/>
		 				<xsl:with-param name="outputNumber" select="$outputNumber"/>
		 			</xsl:call-template>
		 		</xsl:if>
		 		<xsl:if test="not(@indices)">
					<!---->#log "<!---->
					<xsl:value-of select="$outputName"/>
					<!---->" <!---->
					<xsl:value-of select="$outputStartTime"/>
					<xsl:text> </xsl:text>
					<xsl:value-of select="$outputEndTime"/>
					<!----> 0.000001 "%f<!---->
		 			<xsl:call-template name="write_printf_output_all">
                                                <xsl:with-param name="indices" select="number($curr_split_pop/SMLLOWNL:Neuron/@size)"/>
		 			</xsl:call-template>
					<!---->\n" t <!---->
		 			<xsl:call-template name="write_indices_output_all">
                                                <xsl:with-param name="indices" select="number($curr_split_pop/SMLLOWNL:Neuron/@size)"/>
		 				<xsl:with-param name="component" select="$model_file//*[@name=$outputPopName]"/>
		 				<xsl:with-param name="port" select="$outputPortName"/>
		 				<xsl:with-param name="outputNumber" select="$outputNumber"/>
		 			</xsl:call-template>
		 		</xsl:if>
		</xsl:if>
	
	</xsl:for-each>
<!---->

//######################
<!---->

<!-- logs for projections -->

</xsl:for-each>


</xsl:for-each> <!-- END FOR-EACH FOR MODEL XML FILE -->

</xsl:template>

<xsl:include href="SpineML_Component.xsl"/>
<xsl:include href="SpineML_Dynamics.xsl"/>
<xsl:include href="SpineML_Regime.xsl"/>
<xsl:include href="SpineML_StateVariable.xsl"/>
<xsl:include href="SpineML_Parameter.xsl"/>
<xsl:include href="SpineML_AnalogPort.xsl"/>
<xsl:include href="SpineML_EventPort.xsl"/>
<xsl:include href="SpineML_Alias.xsl"/>
<xsl:include href="SpineML_Input.xsl"/>
<xsl:include href="SpineML_Property.xsl"/>
<xsl:include href="SpineML_WeightUpdate.xsl"/>
<xsl:include href="SpineML_Postsynapse.xsl"/>
<xsl:include href="SpineML_Neuron.xsl"/>

</xsl:stylesheet>


