<?xml version="1.0" encoding="ISO-8859-1"?><xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:SMLNL="http://www.shef.ac.uk/SpineMLNetworkLayer" xmlns:SMLLOWNL="http://www.shef.ac.uk/SpineMLLowLevelNetworkLayer" xmlns:SMLCL="http://www.shef.ac.uk/SpineMLComponentLayer"xmlns:fn="http://www.w3.org/2005/xpath-functions">
<xsl:output method="text" version="1.0" encoding="UTF-8" indent="yes"/>

<!-- ################################################
#													#
#		DEFINE ALL PARS / FUNCS FOR PostSynapse  			#
#													#
##################################################-->

<xsl:template match="SMLLOWNL:PostSynapse" mode="defineAll">
<xsl:param name="curr_pop_name"/>
<xsl:param name="split_model_file"/>

<!-- TEMPORARY - THESE SHOULD BE FETCHED FROM THE CONSTRAINTS FILE -->
<xsl:variable name="n_src" select="'100'"/>
<xsl:variable name="n_dst" select="'100'"/>

<xsl:variable name="curr_psp" select="."/>

<!-- GET THE SANATISED VERSION OF THE POPULATION NAME -->
<xsl:variable name="curr_pop_name_sanitised" select="translate($curr_pop_name,' ', 'o')"/>

<!-- PRINT OUT THE CURRENT POSTSYNAPSE NAME --><xsl:text>
</xsl:text>
	<!---->// <xsl:value-of select="@name"/> #########################
<!---->	

<!-- GET A REFERENCE TO THE POSTSYNAPSE COMPONENT -->
<xsl:variable name="linked_psp_file" select="document(@url)"/>

<!-- GENERATE THE UNIQUE SUFFIX NAME FOR THIS POSTSYNAPSE -->
<xsl:variable name="suffix">
	<xsl:value-of select="translate(@name,' -','__')"/><!-- NEW SUFFIX -->
</xsl:variable>
    
    <!-- ADD PARAMETER DEFINES -->
    <!---->// parameters:
<!---->
	<xsl:for-each select="$linked_psp_file//SMLCL:Parameter">
    	<xsl:variable name="par_name" select="@name"/>
		<xsl:choose>
			<xsl:when test="count($curr_psp/SMLNL:Property[@name=$par_name]//SMLNL:FixedValue)=1">
				<!-- NO ARRAY -->
				<!---->float <xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>
				<!-- ADD ASSIGNED VALUE -->
				<xsl:value-of select="concat(' = ', format-number($curr_psp/SMLNL:Property[@name=$par_name]/SMLNL:FixedValue/@value, '0.0#########################'))"/>;
<!---->		</xsl:when>
			<xsl:otherwise>
				<!---->float <xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>]; // (number of dst neurons per core)
<!---->		</xsl:otherwise>
		</xsl:choose>
	</xsl:for-each>
	<!-- ADD STATE VARIABLE DEFINES -->
	<xsl:for-each select="$linked_psp_file//SMLCL:Dynamics">
		<!---->// state variables:
<!---->	<xsl:for-each select="SMLCL:StateVariable">
			<!---->float <xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>]; // (number of dst neurons per core)
<!---->	</xsl:for-each>
	</xsl:for-each>
	<!-- ADD ALIAS AND ANALOG INPUT TEMP VARIABLE DEFINES -->
	<!---->// aliases:
<!---->
	<xsl:for-each select="$linked_psp_file//SMLCL:Alias">
		<!---->float <xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>]; // (number of dst neurons per core)
<!---->
	</xsl:for-each>
	
	<!---->// analog input ports:
<!---->
	<xsl:for-each select="$linked_psp_file//SMLCL:AnalogReceivePort | $linked_psp_file//SMLCL:AnalogReducePort | $linked_psp_file//SMLCL:AnalogSend">
		<!---->float <xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>]; // (number of dst neurons per core)
<!---->
	</xsl:for-each>
	
	<!---->// impulse input ports:
<!---->
	<xsl:for-each select="$linked_psp_file//SMLCL:ImpulseReceivePort">
		<!---->float <xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>]; // (number of dst neurons per core)
<!---->
	</xsl:for-each>
	
	<!-- ADD REGIME SWITCH VARIABLE IF THERE ARE MULTIPLE REGIMES -->
	<!--xsl:if test="count($linked_psp_file//SMLCL:Regime)>1"-->
		<!---->// regime variable:
<!--	 -->int regimeCurr<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->
	<!--/xsl:if-->		

</xsl:template>

<!-- ################################################
#													#
#		INITIALISE ALL DATA FOR PostSynapses				#
#													#
##################################################-->


<xsl:template match="SMLLOWNL:PostSynapse" mode="initData">
<xsl:param name="curr_pop_name"/>
<xsl:param name="split_model_file"/>

<!-- TEMPORARY - THESE SHOULD BE FETCHED FROM THE CONSTRAINTS FILE -->
<xsl:variable name="n_src" select="'100'"/>
<xsl:variable name="n_dst" select="'100'"/>

<xsl:variable name="curr_psp" select="."/>

<!-- GET THE SANATISED VERSION OF THE POPULATION NAME -->
<xsl:variable name="curr_pop_name_sanitised" select="translate($curr_pop_name,' ', 'o')"/>

<!-- PRINT OUT THE CURRENT POSTSYNAPSE NAME --><xsl:text>
</xsl:text>
	<!---->// <xsl:value-of select="@name"/> #########################
<!---->	

<!-- GET A REFERENCE TO THE POSTSYNAPSE COMPONENT -->
<xsl:variable name="linked_psp_file" select="document(@url)"/>

<!-- GENERATE THE UNIQUE SUFFIX NAME FOR THIS POSTSYNAPSE -->
<xsl:variable name="suffix">
	<xsl:value-of select="translate(@name,' -','__')"/><!-- NEW SUFFIX -->
</xsl:variable>
    
    <!-- ADD PARAMETER DEFINES -->
    <!---->	// parameters:
<!---->
	<xsl:for-each select="$linked_psp_file//SMLCL:Parameter">
    	<xsl:variable name="par_name" select="@name"/>
			<!-- WHEN PROBABILITY -->
			<xsl:for-each select="$curr_psp/SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution">
<!---->		<!---->	for (j=0; j &lt; <xsl:value-of select="$n_dst"/>; j += 1) {
<!---->		<!----><xsl:text>		</xsl:text><xsl:value-of select="../@name"/><xsl:value-of select="$suffix"/>[j] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->		<!---->	}
<!---->		</xsl:for-each>
	</xsl:for-each>
	<!-- ADD STATE VARIABLE DEFINES -->
	<xsl:for-each select="$linked_psp_file//SMLCL:Dynamics">
		<!---->	// state variables:
<!---->	<xsl:for-each select="SMLCL:StateVariable">
			<xsl:variable name="par_name" select="@name"/>
			<!-- WHEN FIXED -->
			<xsl:for-each select="$curr_psp/SMLNL:Property[@name=$par_name]//SMLNL:FixedValue">
<!---->		<!---->	for (j=0; j &lt; <xsl:value-of select="$n_dst"/>; j += 1) {
<!---->		<!----><xsl:text>		</xsl:text><xsl:value-of select="../@name"/><xsl:value-of select="$suffix"/>[j] = <xsl:value-of select="@value"/>;
<!---->		<!---->	}
<!---->		</xsl:for-each>
			<!-- WHEN PROBABILITY -->
			<xsl:for-each select="$curr_psp/SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution">
<!---->		<!---->	for (j=0; j &lt; <xsl:value-of select="$n_dst"/>; j += 1) {
<!---->		<!----><xsl:text>		</xsl:text><xsl:value-of select="../@name"/><xsl:value-of select="$suffix"/>[j] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->		<!---->	}
<!---->		</xsl:for-each>
<!---->	</xsl:for-each>
	</xsl:for-each>
	<!-- FILL ANALOG INPUT PIPELINE WITH ZEROS -->
	<xsl:for-each select="$linked_psp_file//SMLCL:ComponentClass">
		<!---->	// analog inputs:
<!---->	<xsl:for-each select="SMLCL:AnalogReceivePort | SMLCL:AnalogReducePort">
<!---->		<!---->	for (j=0; j &lt; <xsl:value-of select="$n_dst"/>; j += 1) {
<!---->		<!----><xsl:text>		</xsl:text><xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>[j] = 0;
<!---->		<!---->	}
<!---->	</xsl:for-each>
	</xsl:for-each>
	<!-- ADD REGIME SWITCH VARIABLE IF THERE ARE MULTIPLE REGIMES -->
	<!--xsl:if test="count($linked_psp_file//SMLCL:Regime)>1"-->
			<!---->// regime variable:
<!---->		<!---->	for (j=0; j &lt; <xsl:value-of select="$n_dst"/>; j += 1) {
<!---->		<!----><xsl:text>		</xsl:text>regimeCurr<xsl:value-of select="$suffix"/>[j] = 1;
<!---->		<!---->	}
<!---->
	<!--/xsl:if-->		

</xsl:template>

<xsl:include href="SpineML_Component.xsl"/>
<xsl:include href="SpineML_Dynamics.xsl"/>
<xsl:include href="SpineML_Regime.xsl"/>
<xsl:include href="SpineML_StateVariable.xsl"/>
<xsl:include href="SpineML_Parameter.xsl"/>
<xsl:include href="SpineML_AnalogPort.xsl"/>
<xsl:include href="SpineML_EventPort.xsl"/>
<xsl:include href="SpineML_Alias.xsl"/>
<xsl:include href="SpineML_Input.xsl"/>
<xsl:include href="SpineML_Property.xsl"/>

</xsl:stylesheet>
